#include "../include/input.h"

/// <summary>
/// Reads a specified configuration data file and evaluates it
/// </summary>
/// <param name="fileName">The file that contains the information to be set</param>
/// <returns>Returns if the prompt should be shown again</returns>
int readInputFile(char *fileName)
{
	char line[1000];
	char keyword[200];
	char value[800];
	FILE *fp;
	
	/* set default data */
	currentConf.data.prompt = HIDE_PROMPT;
	currentConf.data.nx = 16;
	currentConf.data.ny = 16;
	currentConf.data.nz = 1;
	currentConf.data.iseed = 0;
	currentConf.data.betasteps = 1;
	currentConf.data.musteps = 1;
	currentConf.data.betastart = 0.0;
	currentConf.data.mustart = 0.0;
	currentConf.data.betaend = 0.0;
	currentConf.data.muend = 0.0;
	currentConf.data.warmup = 0;
	currentConf.data.iterations = 10000;
	currentConf.data.repetitions = 1;
	currentConf.data.loadlattice = CREATE_NEW_LATTICE;
	currentConf.data.savelattice = FORGET_LATTICE;
	currentConf.data.loadconfig = YES_LOAD_CONFIG;
	currentConf.data.saveeval = NO_SAVE_EVAL;
	currentConf.data.printstatistics = ON;
	sprintf(currentConf.data.loadlatticefile, "def.lattice");
	sprintf(currentConf.data.savelatticefile, "def.lattice");
	sprintf(currentConf.data.loadconfigfile, fileName);
	sprintf(currentConf.data.saveevalfile, "def.evaluation");
	
	/* check if filename seems ok */
	if(strlen(fileName) > MAXFILENAME)
		return INPUT_INV_FILENAME;
		
	/* open the file */
	fp = fopen(fileName, "r");
	
	if(fp != NULL)
	{
		/* read a line */
		while(fgets(line, sizeof(line), fp) != NULL) 
		{
			if(getKeyword(&(line[0]), &(keyword[0]), &(value[0])) != 0 && strlen(value) > 0)
			{
				if(strcmp(keyword, "nx") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.nx));
					
					if(currentConf.data.nx <= 0)
						currentConf.data.nx = 16;
						
					if(currentConf.data.ny > currentConf.data.nx)
						currentConf.data.ny = currentConf.data.nx;
						
					if(currentConf.data.nz > currentConf.data.nx)
						currentConf.data.nz = currentConf.data.nx;
				}
				else if(strcmp(keyword, "ny") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.ny));
					
					if(currentConf.data.ny <= 0 || currentConf.data.ny > currentConf.data.nx)
						currentConf.data.ny = 16 > currentConf.data.nx ? currentConf.data.nx : 16;
						
					if(currentConf.data.nz > currentConf.data.ny)
						currentConf.data.nz = currentConf.data.ny;
				}
				else if(strcmp(keyword, "nz") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.nz));
					
					if(currentConf.data.nz <= 0 || currentConf.data.nz > currentConf.data.ny)
						currentConf.data.nz = 1;
				}
				else if(strcmp(keyword, "iseed") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.iseed));
				}
				else if(strcmp(keyword, "betasteps") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.betasteps));
					
					if(currentConf.data.betasteps <= 0)
						currentConf.data.betasteps = 1;
				}
				else if(strcmp(keyword, "musteps") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.musteps));
					
					if(currentConf.data.musteps <= 0)
						currentConf.data.musteps = 1;
				}
				else if(strcmp(keyword, "betastart") == 0)
				{
					sscanf(value, "%lf", &(currentConf.data.betastart));
					
					if(currentConf.data.betastart < 0.0)
						currentConf.data.betastart = 0.0;
				}
				else if(strcmp(keyword, "mustart") == 0)
				{
					sscanf(value, "%lf", &(currentConf.data.mustart));
				}
				else if(strcmp(keyword, "betaend") == 0)
				{
					sscanf(value, "%lf", &(currentConf.data.betaend));
					
					if(currentConf.data.betaend < 0.0)
						currentConf.data.betaend = currentConf.data.betastart;
				}
				else if(strcmp(keyword, "muend") == 0)
				{
					sscanf(value, "%lf", &(currentConf.data.muend));
				}
				else if(strcmp(keyword, "warmup") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.warmup));
					
					if(currentConf.data.warmup < 0)
						currentConf.data.warmup = 0;
				}
				else if(strcmp(keyword, "iterations") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.iterations));
					
					if(currentConf.data.iterations <= 0)
						currentConf.data.iterations = 10000;
				}
				else if(strcmp(keyword, "repetitions") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.repetitions));
					
					if(currentConf.data.repetitions <= 0)
						currentConf.data.repetitions = 1;
				}
				else if(strcmp(keyword, "loadlattice") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.loadlattice));
					
					if(currentConf.data.loadlattice < CREATE_NEW_LATTICE || currentConf.data.loadlattice > LOAD_LATTICE_TEXT)
						currentConf.data.loadlattice = CREATE_NEW_LATTICE;
				}
				else if(strcmp(keyword, "savelattice") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.savelattice));
					
					if(currentConf.data.savelattice < FORGET_LATTICE || currentConf.data.savelattice > SAVE_LATTICE_TEXT)
						currentConf.data.savelattice = FORGET_LATTICE;
				}
				else if(strcmp(keyword, "saveeval") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.saveeval));
					
					if(currentConf.data.saveeval < NO_SAVE_EVAL || currentConf.data.saveeval > YES_SAVE_EVAL)
						currentConf.data.saveeval = NO_SAVE_EVAL;
				}
				else if(strcmp(keyword, "loadlatticefile") == 0)
				{
					sscanf(value, "%s", currentConf.data.savelatticefile);
				}
				else if(strcmp(keyword, "savelatticefile") == 0)
				{
					sscanf(value, "%s", currentConf.data.savelatticefile);
				}
				else if(strcmp(keyword, "saveevalfile") == 0)
				{
					sscanf(value, "%s", currentConf.data.saveevalfile);
				}
				else if(strcmp(keyword, "printstatistics") == 0)
				{
					sscanf(value, "%d", &(currentConf.data.printstatistics));
					
					if(currentConf.data.printstatistics < OFF || currentConf.data.printstatistics > ON)
						currentConf.data.printstatistics = ON;
				}
			}
		}
	}
	else /* why isnt the file open?! */
		return INPUT_FILE_NOT_FOUND;
	
	/* close everything and return no error */
	fclose(fp);
	return INPUT_NO_ERROR;
}

/// <summary>
/// Gets the keyword, value pair of the line that is currently read in
/// </summary>
/// <param name="line">The line where the keyword / value pair should be in</param>
/// <param name="keyword">The extracted keyword value</param>
/// <param name="value">The extracted value value</param>
/// <returns>Returns 0 if no Keyword / value pair was found - else the position of the = char</returns>
int getKeyword(char* line, char* keyword, char* value)
{
	int i;
	char tmp[800];
	
	for(i = 0; i < strlen(line); i++)
	{
		if(strncmp(line + i, "=", 1) == 0)
		{
			extract(line, tmp, keyword, 0, i > 199 ? 199 : i);
			extract(line, tmp, value, i + 1, 998 - i > 799 ? 799 : 998 - i);
			return i;
		}
	}
			
	return 0;
}

/// <summary>
/// Extracts part of a string s into a string d
/// </summary>
/// <param name="s">The string where a substring should be extracted from</param>
/// <param name="t">A temporary string to be used</param>
/// <param name="d">Our destination string with the substring chars</param>
/// <param name="pos">The position where the substring should start</param>
/// <param name="len">The length - how many chars should the substring contain?</param>
void extract(char *s, char *t, char *d, int pos, int len)
{
	s = s + pos;
	t = s + len;
	while(s != t)
	{
		*d = *s;
		s++;
		d++;
	}
	*d = '\0';
}

/// <summary>
/// Evaluates the command line arguments and shows then either
/// the prompt or reads in the specified configuration file
/// </summary>
/// <param name="argc">number of command line arguments</param>
/// <param name="argv">command line arguments as strings</param>
/// <returns>If the input prompt should be shown</returns>
int commandLineParser(int argc, char* argv[])
{
	int i;
	
	if(thisNode.globalIndex == 0)
	{
		for(i = 1; i < argc; i++)
		{
			/* if we have the -c */
			if(strcmp(argv[i],"-c") == 0)
			{
				/* but nothing else specified */
				if(i == argc - 1)
				{
					printf("A configuration file must be specified using -c.\n");
					terminate(EXIT_CMD_LINE);
				}

				/* or we probably have something */
				if(readInputFile(argv[i + 1]) == INPUT_NO_ERROR)
				{
					distributeInputData();
					return HIDE_PROMPT;
				}
				else
				{
					printf("An error occured while reading the specified configuration file.\n");
					terminate(EXIT_FILE_ERR);
				}
			}
			/* if we have some help statement */
			else if(strcmp(argv[i],"-h") == 0)
			{
				printf("Command line arguments\n");
				printf("---\n");
				printf("-c file\t uses the configuration specified in the file\n");
				printf("-l file\t displays the header information of the given lattice\n");
				printf("-? \t displays this help\n");
				printf("---\n");
				terminate(EXIT_OK);
			}
			else if(strcmp(argv[i],"-l") == 0)
			{
				/* but nothing else specified */
				if(i == argc - 1)
				{
					printf("A lattice file must be specified using -l.\n");
					terminate(EXIT_CMD_LINE);
				} 
				else
				{				
					readLatticeHeader(argv[i + 1], OFF);
					terminate(EXIT_OK);
				}
			}
		}
	}
	
	return inputPrompt();
}

/// <summary>
/// Shows the input prompt to the user - asking for configuration
/// </summary>
/// <returns>If the input prompt should be shown</returns>
int inputPrompt()
{
	char buff[MAXFILENAME];
	currentConf.data.prompt = HIDE_PROMPT;
	
	if(thisNode.globalIndex == 0)
	{
		printf("Please enter the configuration parameters [type, default value, condition]\n");
		printf("Just pressing the enter key will result in taking the def. value.\n");
		printf("-----\n\n");
		printf("loadconfig [int, 0, FLAG: 0=NO 1=LOAD W NO PROMPT 2=LOAD W PROMPT] = ");
		gets(buff);
		
		if(strlen(buff) == 0)
		{
			currentConf.data.loadconfig = NO_LOAD_CONFIG;
		}
		else
		{
			sscanf(buff, "%d", &(currentConf.data.loadconfig));
			
			if(currentConf.data.loadconfig < NO_LOAD_CONFIG || currentConf.data.loadconfig > YES_PROMPT_MODE)
				currentConf.data.loadconfig = NO_LOAD_CONFIG;
		}
		
		if(currentConf.data.loadconfig != NO_LOAD_CONFIG)
		{
			if(currentConf.data.loadconfig == YES_PROMPT_MODE)
				currentConf.data.prompt = SHOW_PROMPT;
				
			printf("loadconfigfile [string, def.config, -] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				sprintf(currentConf.data.loadconfigfile, "def.config");
			else
				sscanf(buff, "%s", currentConf.data.loadconfigfile);
				
			if(readInputFile(currentConf.data.loadconfigfile) != INPUT_NO_ERROR)
				return 0;
		}
		else
		{
			sprintf(currentConf.data.loadconfigfile, "def.config");

			printf("nx [int, 16, >0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.nx = 16;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.nx));
				
				if(currentConf.data.nx <= 0)
					currentConf.data.nx = 16;
			}
			
			printf("ny [int, 16, <=nx>0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.ny = 16 > currentConf.data.nx ? currentConf.data.nx : 16;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.ny));
				
				if(currentConf.data.ny <= 0 || currentConf.data.ny > currentConf.data.nx)
					currentConf.data.ny = 16 > currentConf.data.nx ? currentConf.data.nx : 16;
			}
			
			printf("nz [int, 1, <=ny>0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.nz = 1;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.nz));
				
				if(currentConf.data.nz <= 0 || currentConf.data.nz > currentConf.data.ny)
					currentConf.data.nz = 1;
			}
			
			printf("iseed [int, 0, -] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.iseed = 0;
			else
				sscanf(buff, "%d", &(currentConf.data.iseed));
			
			printf("betastart [double, 0, >=0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.betastart = 0.0;
			else
			{
				sscanf(buff, "%lf", &(currentConf.data.betastart));
				
				if(currentConf.data.betastart < 0.0)
					currentConf.data.betastart = 0.0;
			}
				
			printf("betaend [double, 0, >=0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.betaend = currentConf.data.betastart;
			else
			{
				sscanf(buff, "%lf", &(currentConf.data.betaend));
				
				if(currentConf.data.betaend < 0.0)
					currentConf.data.betaend = currentConf.data.betastart;
			}
				
			printf("betasteps [int, 1, >0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.betasteps = 1;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.betasteps));
				
				if(currentConf.data.betasteps <= 0)
					currentConf.data.betasteps = 1;
			}
			
			printf("mustart [double, 0, -] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.mustart = 0.0;
			else
				sscanf(buff, "%lf", &(currentConf.data.mustart));
			
			printf("muend [double, 0, -] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.muend = currentConf.data.mustart;
			else
				sscanf(buff, "%lf", &(currentConf.data.muend));
			
			printf("musteps [int, 1, >0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.musteps = 1;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.musteps));
				
				if(currentConf.data.musteps <= 0)
					currentConf.data.musteps = 1;
			}
				
			printf("warmup [int, 0, >=0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.warmup = 0;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.warmup));
				
				if(currentConf.data.warmup < 0)
					currentConf.data.warmup = 0;
			}
			
			printf("iterations [int, 10000, >0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.iterations = 10000;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.iterations));
				
				if(currentConf.data.iterations <= 0)
					currentConf.data.iterations = 10000;
			}
			
			printf("repetitions [int, 1, >0] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.repetitions = 1;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.repetitions));
				
				if(currentConf.data.repetitions <= 0)
					currentConf.data.repetitions = 1;
			}
			
			printf("loadlattice [int, 0, FLAG: 0=CREATE NEW 1=LOAD BINARY 2=LOAD TEXT] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.loadlattice = CREATE_NEW_LATTICE;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.loadlattice));
				
				if(currentConf.data.loadlattice < CREATE_NEW_LATTICE || currentConf.data.loadlattice > LOAD_LATTICE_TEXT)
					currentConf.data.loadlattice = CREATE_NEW_LATTICE;
			}
			
			if(currentConf.data.loadlattice != CREATE_NEW_LATTICE)
			{
				printf("loadlatticefile [string, def.lattice, -] = ");
				gets(buff);
				
				if(strlen(buff) == 0)
					sprintf(currentConf.data.loadlatticefile, "def.lattice");
				else
					sscanf(buff, "%s", currentConf.data.loadlatticefile);
			}
			else
				sprintf(currentConf.data.loadlatticefile, "def.lattice");
			
			printf("savelattice [int, 0, FLAG: 0=FORGET 1=SAVE DEFAULT 2=SAVE BINARY 3=SAVE TEXT] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.savelattice = FORGET_LATTICE;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.savelattice));
				
				if(currentConf.data.savelattice < FORGET_LATTICE || currentConf.data.savelattice > SAVE_LATTICE_TEXT)
					currentConf.data.savelattice = FORGET_LATTICE;
			}
			
			if(currentConf.data.savelattice == SAVE_LATTICE_DEFAULT)
			{
				if(currentConf.data.loadlattice == 0)
					sprintf(currentConf.data.savelatticefile, "def.lattice");
				else
					sprintf(currentConf.data.savelatticefile, currentConf.data.loadlatticefile);
			}
			else if(currentConf.data.savelattice != FORGET_LATTICE)
			{
				printf("savelatticefile [string, def.lattice, -] = ");
				gets(buff);
				
				if(strlen(buff) == 0)
					sprintf(currentConf.data.savelatticefile, "def.lattice");
				else
					sscanf(buff, "%s", currentConf.data.savelatticefile);
			}
			else
				sprintf(currentConf.data.savelatticefile, "def.lattice");
			
			printf("saveeval [int, 0, FLAG: 0=NO 1=YES] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.saveeval = NO_SAVE_EVAL;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.saveeval));
				
				if(currentConf.data.saveeval < NO_SAVE_EVAL || currentConf.data.saveeval > YES_SAVE_EVAL)
					currentConf.data.saveeval = NO_SAVE_EVAL;
			}
			
			if(currentConf.data.saveeval != NO_SAVE_EVAL)
			{
				printf("saveevalfile [string, def.evaluation, -] = ");
				gets(buff);
				
				if(strlen(buff) == 0)
					sprintf(currentConf.data.saveevalfile, "defevaluation");
				else
					sscanf(buff, "%s", currentConf.data.saveevalfile);
			}
			else
				sprintf(currentConf.data.saveevalfile, "def.eval");
				
			printf("printstatistics [int, 1, FLAG: 0=NO 1=YES] = ");
			gets(buff);
			
			if(strlen(buff) == 0)
				currentConf.data.printstatistics = ON;
			else
			{
				sscanf(buff, "%d", &(currentConf.data.printstatistics));
				
				if(currentConf.data.printstatistics < OFF || currentConf.data.printstatistics > ON)
					currentConf.data.printstatistics = ON;
			}
			
			printf("show prompt (for setting a new configuration) after program run? (Y/N): ");
			gets(buff);
			
			currentConf.data.prompt = (strcmp(buff,"Y") == 0 || strcmp(buff,"y") == 0) ? SHOW_PROMPT : HIDE_PROMPT;
		}
		
		distributeInputData();
	}
	else
		distributeInputData();
	
	return currentConf.data.prompt;
}

