#include "../include/utils.h"

/// <summary>
/// Every node can send a print statement to this function and only the global Master will print it out
/// </summary>
/// <param name="string">The string that should be printed only by globalIndex == 0</param>
/// <param name="...">Additional arguments that fill the %d etc. of the string</param>
void masterPrint(const char* string, ... )
{
	struct tm *ptr;
	time_t lt;
	char* tstr;
	
	if(thisNode.globalIndex == 0)
	{
		lt = time(NULL);
		ptr = localtime(&lt);
		tstr = asctime(ptr);
		tstr[strlen(tstr) - 1] = '\0';
        
        va_list args;
        fprintf( stderr, "%s | ", tstr );
        va_start( args, string );
        vfprintf( stderr, string, args );
        va_end( args );
        fprintf( stderr, "\n" );
	}
}

/// <summary>
/// This function can be used in order to specify a rank of the node and a print statement that can be used for debug printing
/// This function will only execute the print code when the DEBUGPRINT macro is on - i.e. when we use a debug app
/// </summary>
/// <param name="master">The rank that the node needs to have in order to print the content (its a flag!)</param>
/// <param name="string">The string that should be printed only by nodes of the rank specified</param>
/// <param name="...">Additional arguments that fill the %d etc. of the string</param>
void debugPrint(int master, const char* string, ... )
{
#ifdef DEBUGPRINT
    if(master == 0)
    {
        if(thisNode.globalIndex == 0)
        {
            va_list args;
            fprintf( stderr, "GDEBUG: " );
            va_start( args, string );
            vfprintf( stderr, string, args );
            va_end( args );
            fprintf( stderr, "\n" );
        }
    }
    else if(master == 1)
    {
        if(thisNode.nodeIndex == 0)
        {
            va_list args;
            fprintf( stderr, "LDEBUG: " );
            va_start( args, string );
            vfprintf( stderr, string, args );
            va_end( args );
            fprintf( stderr, "\n" );
        }
    }
    else
    {
        va_list args;
        fprintf( stderr, "ADEBUG: " );
        va_start( args, string );
        vfprintf( stderr, string, args );
        va_end( args );
        fprintf( stderr, "\n" );
    }
#endif
}
