#include "../include/vector.h"

/// <summary>
/// Executes the reduce operation in form of a sum with a given vector
/// </summary>
/// <param name="vec">The vector that goes into the reduce operation</param>
/// <param name="out">The vector that comes out of the reduce operation</param>
/// <param name="n">The length of the vector that goes in and out</param>
void vectorsReduce(double* vec, double* out, int n)
{
#ifdef MPICOMM
	// execute MPI code
	MPI_Reduce(vec, out, n, MPI_DOUBLE, MPI_SUM, 0, thisNode.MPI_COMM_MASTERS);
    //debugPrint(GLOBALM, "makeGlobalStatistic: MPI_Reduce done on lattice %i", currentConf.latticeIndex);
#else
	int i;
	
	for(i = 0; i < n; i++)
		out[i] = vec[i];
#endif /*MPICOMM*/	
}

/// <summary>
/// Executes the reduce operation in form of a sum with a given vector and performs two summations of this vector
/// </summary>
/// <param name="vec">The vector that goes into the reduce operation</param>
/// <param name="sum">The reference to the simple sum variable (sum_i v_i)</param>
/// <param name="sumsq">The reference to the squares sum variable (sum_i v_i^2)</param>
void vectorsReduceSum(double* vec, double* sum, double* sumsq)
{
#ifdef MPICOMM
	double out;
	/* reduce vectors from all sites */
	MPI_Reduce(vec, &out, 1, MPI_DOUBLE, MPI_SUM, 0, thisNode.MPI_COMM_BLADE);
    //debugPrint(LATTICEM, "updateLatticeStatistic MPI_Reduce done on lattice %i", currentConf.latticeIndex);
	
	/* sum on root node of each blade */
	if(thisNode.nodeIndex == 0)
	{
		*sum = out;
		*sumsq = out * out;
	}		
#else
	/* execute single processor code */
	*sum = *vec;
	*sumsq = *vec * *vec;
#endif /*MPICOMM*/ 
}